@@ -3,7 +3,7 @@ from binarybotapi.api import IQOptionAPI
 import binarybotapi.constants as OP_code
 import binarybotapi.country_id as Country
 import threading
-import time
+import time, json
 import logging
 import operator
 import binarybotapi.global_value as global_value
@@ -11,6 +11,7 @@ from collections import defaultdict
 from collections import deque
 from binarybotapi.expiration import get_expiration_time, get_remaning_time
 from datetime import datetime, timedelta
+from random import randint
 
 
 def nested_dict(n, type):
@@ -23,7 +24,7 @@ def nested_dict(n, type):
 class IQ_Option:
     __version__ = "6.8.9.1"
 
-    def __init__(self, email, password):
+    def __init__(self, email, password, active_account_type="PRACTICE"):
         self.size = [1, 5, 10, 15, 30, 60, 120, 300, 600, 900, 1800,
                      3600, 7200, 14400, 28800, 43200, 86400, 604800, 2592000]
         self.email = email
@@ -33,6 +34,7 @@ class IQ_Option:
         self.subscribe_candle = []
         self.subscribe_candle_all_size = []
         self.subscribe_mood = []
+        self.subscribe_indicators = []
         # for digit
         self.get_digital_spot_profit_after_sale_data = nested_dict(2, int)
         self.get_realtime_strike_list_temp_data = {}
@@ -41,7 +43,6 @@ class IQ_Option:
             "User-Agent": r"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.139 Safari/537.36"}
         self.SESSION_COOKIE = {}
         #
-
         # --start
         # self.connect()
         # this auto function delay too long
@@ -75,7 +76,7 @@ class IQ_Option:
         self.SESSION_HEADER = header
         self.SESSION_COOKIE = cookie
 
-    def connect(self):
+    def connect(self, sms_code=None):
         try:
             self.api.close()
         except:
@@ -85,7 +86,18 @@ class IQ_Option:
         self.api = IQOptionAPI(
             "iqoption.com", self.email, self.password)
         check = None
-        self.api.set_session(headers=self.SESSION_HEADER, cookies=self.SESSION_COOKIE)
+
+        # 2FA--
+        if sms_code is not None:
+            self.api.setTokenSMS(self.resp_sms)
+            status, reason = self.api.connect2fa(sms_code)
+            if not status:
+                return status, reason
+        # 2FA--
+
+        self.api.set_session(headers=self.SESSION_HEADER,
+                             cookies=self.SESSION_COOKIE)
+
         check, reason = self.api.connect()
 
         if check == True:
@@ -95,7 +107,10 @@ class IQ_Option:
             # ---------for async get name: "position-changed", microserviceName
             while global_value.balance_id == None:
                 pass
-            self.position_change_all("subscribeMessage", global_value.balance_id)
+
+            self.position_change_all(
+                "subscribeMessage", global_value.balance_id)
+
             self.order_changed_all("subscribeMessage")
             self.api.setOptions(1, True)
 
@@ -122,10 +137,22 @@ class IQ_Option:
             # self.get_balance_id()
             return True, None
         else:
+            if json.loads(reason)['code'] == 'verify':
+                response = self.api.send_sms_code(json.loads(reason)['token'])
+
+                if response.json()['code'] != 'success':
+                    return False, response.json()['message']
+
+                # token_sms
+                self.resp_sms = response
+                return False, "2FA"
             return False, reason
 
     # self.update_ACTIVES_OPCODE()
 
+    def connect_2fa(self, sms_code):
+        return self.connect(sms_code=sms_code)
+
     def check_connect(self):
         # True/False
 
@@ -163,13 +190,13 @@ class IQ_Option:
             pass
         return self.api.financial_information
 
-    def get_leader_board(self, country, from_position, to_position, near_traders_count, user_country_id=0,
-                         near_traders_country_count=0, top_country_count=0, top_count=0, top_type=2):
+    def get_leader_board(self, country, from_position, to_position, near_traders_count, user_country_id=0, near_traders_country_count=0, top_country_count=0, top_count=0, top_type=2):
         self.api.leaderboard_deals_client = None
 
         country_id = Country.ID[country]
-        self.api.Get_Leader_Board(country_id, user_country_id, from_position, to_position, near_traders_country_count,
-                                  near_traders_count, top_country_count, top_count, top_type)
+        self.api.Get_Leader_Board(country_id, user_country_id, from_position, to_position,
+                                  near_traders_country_count, near_traders_count, top_country_count, top_count, top_type)
+
         while self.api.leaderboard_deals_client == None:
             pass
         return self.api.leaderboard_deals_client
@@ -204,7 +231,7 @@ class IQ_Option:
         for dirr in (["binary", "turbo"]):
             for i in init_info["result"][dirr]["actives"]:
                 OP_code.ACTIVES[(init_info["result"][dirr]
-                ["actives"][i]["name"]).split(".")[1]] = int(i)
+                                 ["actives"][i]["name"]).split(".")[1]] = int(i)
 
     # _________________________self.api.get_api_option_init_all() wss______________________
     def get_all_init(self):
@@ -225,12 +252,12 @@ class IQ_Option:
                     logging.error('**warning** get_all_init late 30 sec')
                     break
                 try:
-                    if self.api.api_option_init_all_result != None:
+                    if self.api.api_option_init_all_result is not None:
                         break
                 except:
                     pass
             try:
-                if self.api.api_option_init_all_result["isSuccessful"] == True:
+                if self.api.api_option_init_all_result["isSuccessful"]:
                     return self.api.api_option_init_all_result
             except:
                 pass
@@ -238,9 +265,12 @@ class IQ_Option:
     def get_all_init_v2(self):
         self.api.api_option_init_all_result_v2 = None
 
+        if not self.check_connect():
+            self.connect()
+
         self.api.get_api_option_init_all_v2()
         start_t = time.time()
-        while self.api.api_option_init_all_result_v2 == None:
+        while self.api.api_option_init_all_result_v2 is None:
             if time.time() - start_t >= 30:
                 logging.error('**warning** get_all_init_v2 late 30 sec')
                 return None
@@ -256,47 +286,56 @@ class IQ_Option:
         binary_data = self.get_all_init_v2()
         binary_list = ["binary", "turbo"]
         for option in binary_list:
-            binaryOptionData = binary_data.get(option)
-            if binaryOptionData is not None:
-                for actives_id in binaryOptionData.get("actives"):
-                    active = binaryOptionData.get("actives").get(actives_id)
-                    if active is not None:
-                        dottedName = str(active.get("name"))
-                        if dottedName is not None:
-                            name = dottedName.split(".")[1]
-                            if active.get("enabled"):
-                                if active.get("is_suspended"):
-                                    OPEN_TIME[option][name]["open"] = False
+            if binary_data is not None:
+                binaryOptionData = binary_data.get(option)
+                if binaryOptionData is not None:
+                    for actives_id in binaryOptionData.get("actives"):
+                        active = binaryOptionData.get("actives").get(actives_id)
+                        if active is not None:
+                            dottedName = str(active.get("name"))
+                            if dottedName is not None:
+                                name = dottedName.split(".")[1]
+                                if active.get("enabled"):
+                                    if active.get("is_suspended"):
+                                        OPEN_TIME[option][name]["open"] = False
+                                    else:
+                                        OPEN_TIME[option][name]["open"] = True
                                 else:
-                                    OPEN_TIME[option][name]["open"] = True
-                            else:
-                                OPEN_TIME[option][name]["open"] = active.get("enabled")
+                                    OPEN_TIME[option][name]["open"] = active.get("enabled")
 
         # for digital
-        digital_data = self.get_digital_underlying_list_data()["underlying"]
-        for digital in digital_data:
-            name = digital["underlying"]
-            schedule = digital["schedule"]
-            OPEN_TIME["digital"][name]["open"] = False
-            for schedule_time in schedule:
-                start = schedule_time["open"]
-                end = schedule_time["close"]
-                if start < time.time() < end:
-                    OPEN_TIME["digital"][name]["open"] = True
+        digData = self.get_digital_underlying_list_data()
+        if digData is not None:
+            digital_data = digData.get("underlying")
+            if digital_data is not None:
+                for digital in digital_data:
+                    name = digital.get("underlying")
+                    schedule = digital.get("schedule")
+                    OPEN_TIME["digital"][name]["open"] = False
+                    for schedule_time in schedule:
+                        if schedule_time is not None:
+                            start = schedule_time.get("open")
+                            end = schedule_time.get("close")
+                            if start < time.time() < end:
+                                OPEN_TIME["digital"][name]["open"] = True
 
         # for OTHER
         instrument_list = ["cfd", "forex", "crypto"]
         for instruments_type in instrument_list:
-            ins_data = self.get_instruments(instruments_type)["instruments"]
-            for detail in ins_data:
-                name = detail["name"]
-                schedule = detail["schedule"]
-                OPEN_TIME[instruments_type][name]["open"] = False
-                for schedule_time in schedule:
-                    start = schedule_time["open"]
-                    end = schedule_time["close"]
-                    if start < time.time() < end:
-                        OPEN_TIME[instruments_type][name]["open"] = True
+            instrumentData = self.get_instruments(instruments_type)
+            if instrumentData is not None:
+                ins_data = instrumentData.get("instruments")
+                for detail in ins_data:
+                    if detail is not None:
+                        name = detail.get("name")
+                        schedule = detail.get("schedule")
+                        OPEN_TIME[instruments_type][name]["open"] = False
+                        for schedule_time in schedule:
+                            if schedule_time is not None:
+                                start = schedule_time.get("open")
+                                end = schedule_time.get("close")
+                                if start < time.time() < end:
+                                    OPEN_TIME[instruments_type][name]["open"] = True
 
         return OPEN_TIME
 
@@ -323,17 +362,17 @@ class IQ_Option:
             name = init_info["result"]["turbo"]["actives"][actives]["name"]
             name = name[name.index(".") + 1:len(name)]
             all_profit[name]["turbo"] = (
-                                                100.0 -
-                                                init_info["result"]["turbo"]["actives"][actives]["option"]["profit"][
-                                                    "commission"]) / 100.0
+                100.0 -
+                init_info["result"]["turbo"]["actives"][actives]["option"]["profit"][
+                    "commission"]) / 100.0
 
         for actives in init_info["result"]["binary"]["actives"]:
             name = init_info["result"]["binary"]["actives"][actives]["name"]
             name = name[name.index(".") + 1:len(name)]
             all_profit[name]["binary"] = (
-                                                 100.0 -
-                                                 init_info["result"]["binary"]["actives"][actives]["option"]["profit"][
-                                                     "commission"]) / 100.0
+                100.0 -
+                init_info["result"]["binary"]["actives"][actives]["option"]["profit"][
+                    "commission"]) / 100.0
         return all_profit
 
     # ----------------------------------------
@@ -348,10 +387,10 @@ class IQ_Option:
     """def get_profile(self):
         while True:
             try:
-                 
+
                 respon = self.api.getprofile().json()
                 time.sleep(self.suspend)
-            
+
                 if respon["isSuccessful"] == True:
                     return respon
             except:
@@ -397,13 +436,16 @@ class IQ_Option:
     def get_balance_mode(self):
         # self.api.profile.balance_type=None
         profile = self.get_profile_ansyc()
-        for balance in profile["balances"]:
+        for balance in profile.get("balances"):
             if balance["id"] == global_value.balance_id:
                 if balance["type"] == 1:
                     return "REAL"
                 elif balance["type"] == 4:
                     return "PRACTICE"
 
+                elif balance["type"] == 2:
+                    return "TOURNAMENT"
+
     def reset_practice_balance(self):
         self.api.training_balance_reset_request = None
         self.api.reset_training_balance()
@@ -412,20 +454,24 @@ class IQ_Option:
         return self.api.training_balance_reset_request
 
     def position_change_all(self, Main_Name, user_balance_id):
-        instrument_type = ["cfd", "forex", "crypto", "digital-option", "turbo-option", "binary-option"]
+        instrument_type = ["cfd", "forex", "crypto",
+                           "digital-option", "turbo-option", "binary-option"]
         for ins in instrument_type:
-            self.api.portfolio(Main_Name=Main_Name, name="portfolio.position-changed", instrument_type=ins,
-                               user_balance_id=user_balance_id)
+            self.api.portfolio(Main_Name=Main_Name, name="portfolio.position-changed",
+                               instrument_type=ins, user_balance_id=user_balance_id)
 
     def order_changed_all(self, Main_Name):
-        instrument_type = ["cfd", "forex", "crypto", "digital-option", "turbo-option", "binary-option"]
+        instrument_type = ["cfd", "forex", "crypto",
+                           "digital-option", "turbo-option", "binary-option"]
         for ins in instrument_type:
-            self.api.portfolio(Main_Name=Main_Name, name="portfolio.order-changed", instrument_type=ins)
+            self.api.portfolio(
+                Main_Name=Main_Name, name="portfolio.order-changed", instrument_type=ins)
 
     def change_balance(self, Balance_MODE):
         def set_id(b_id):
             if global_value.balance_id != None:
-                self.position_change_all("unsubscribeMessage", global_value.balance_id)
+                self.position_change_all(
+                    "unsubscribeMessage", global_value.balance_id)
 
             global_value.balance_id = b_id
 
@@ -433,6 +479,7 @@ class IQ_Option:
 
         real_id = None
         practice_id = None
+        tournament_id = None
 
         for balance in self.get_profile_ansyc()["balances"]:
             if balance["type"] == 1:
@@ -440,13 +487,18 @@ class IQ_Option:
             if balance["type"] == 4:
                 practice_id = balance["id"]
 
+            if balance["type"] == 2:
+                tournament_id = balance["id"]
+
         if Balance_MODE == "REAL":
             set_id(real_id)
 
         elif Balance_MODE == "PRACTICE":
-
             set_id(practice_id)
 
+        elif Balance_MODE == "TOURNAMENT":
+            set_id(tournament_id)
+
         else:
             logging.error("ERROR doesn't have this mode")
             exit(1)
@@ -641,22 +693,23 @@ class IQ_Option:
 
     # -----------------traders_mood----------------------
 
-    def start_mood_stream(self, ACTIVES):
+    def start_mood_stream(self, ACTIVES, instrument="turbo-option"):
         if ACTIVES in self.subscribe_mood == False:
             self.subscribe_mood.append(ACTIVES)
 
         while True:
-            self.api.subscribe_Traders_mood(OP_code.ACTIVES[ACTIVES])
+            self.api.subscribe_Traders_mood(
+                OP_code.ACTIVES[ACTIVES], instrument)
             try:
                 self.api.traders_mood[OP_code.ACTIVES[ACTIVES]]
                 break
             except:
                 time.sleep(5)
 
-    def stop_mood_stream(self, ACTIVES):
+    def stop_mood_stream(self, ACTIVES, instrument="turbo-option"):
         if ACTIVES in self.subscribe_mood == True:
             del self.subscribe_mood[ACTIVES]
-        self.api.unsubscribe_Traders_mood(OP_code.ACTIVES[ACTIVES])
+        self.api.unsubscribe_Traders_mood(OP_code.ACTIVES[ACTIVES], instrument)
 
     def get_traders_mood(self, ACTIVES):
         # return highter %
@@ -666,7 +719,28 @@ class IQ_Option:
         # return highter %
         return self.api.traders_mood
 
-    ##############################################################################################
+##############################################################################################
+
+    # -----------------technical_indicators----------------------
+
+    def get_technical_indicators(self, ACTIVES):
+        request_id = self.api.get_Technical_indicators(
+            OP_code.ACTIVES[ACTIVES])
+        while self.api.technical_indicators.get(request_id) == None:
+            pass
+        return self.api.technical_indicators[request_id]
+
+##############################################################################################
+
+
+##############################################################################################
+
+    def check_binary_order(self, order_id):
+        while order_id not in self.api.order_binary:
+            pass
+        your_order = self.api.order_binary[order_id]
+        del self.api.order_binary[order_id]
+        return your_order
 
     def check_win(self, id_number):
         # 'win':win money 'equal':no win no loose   'loose':loose money
@@ -693,17 +767,26 @@ class IQ_Option:
                     pass
             time.sleep(polling_time)
 
-    def check_win_v3(self, id_number):
+        # Function by kkagill ( https://github.com/Lu-Yi-Hsun/iqoptionapi/issues/196 | https://github.com/kkagill )
+        # Function only work with Options!
+
+    def check_win_v4(self, id_number):
         while True:
             try:
-
-                if self.get_async_order(id_number)["option-closed"] != {}:
+                if self.api.socket_option_closed[id_number] != None:
                     break
             except:
                 pass
+        x = self.api.socket_option_closed[id_number]
+        return x['msg']['win'], (0 if x['msg']['win'] == 'equal' else float(x['msg']['sum']) * -1 if x['msg']['win'] == 'loose' else float(x['msg']['win_amount']) - float(x['msg']['sum']))
 
-        return self.get_async_order(id_number)["option-closed"]["msg"]["profit_amount"] - \
-               self.get_async_order(id_number)["option-closed"]["msg"]["amount"]
+    def check_win_v3(self, id_number):
+        while True:
+            result = self.get_optioninfo_v2(10)
+            if result['msg']['closed_options'][0]['id'][0] == id_number and result['msg']['closed_options'][0]['id'][0] != None:
+                return result['msg']['closed_options'][0]['win'], (result['msg']['closed_options'][0]['win_amount']-result['msg']['closed_options'][0]['amount'] if result['msg']['closed_options'][0]['win'] != 'equal' else 0)
+                break
+            time.sleep(1)
 
     # -------------------get infomation only for binary option------------------------
 
@@ -817,12 +900,14 @@ class IQ_Option:
         self.api.buy_successful = None
         self.api.resultInfo = None
         req_id = "buy"
+        # req_id = "buy"
+        req_id = str(randint(0, 10000))
         try:
             self.api.buy_multi_option[req_id]["id"] = None
         except:
             pass
         self.api.buyv3(
-            price, OP_code.ACTIVES[ACTIVES], ACTION, expirations, req_id)
+            float(price), OP_code.ACTIVES[ACTIVES], str(ACTION), int(expirations), req_id)
         start_t = time.time()
         id = None
         self.api.result = None
@@ -836,7 +921,7 @@ class IQ_Option:
                 id = self.api.buy_multi_option[req_id]["id"]
             except:
                 pass
-            if time.time() - start_t >= 5:
+            if time.time() - start_t >= 30:
                 logging.error('**warning** buy late 5 sec')
                 return False, None
 
@@ -849,7 +934,13 @@ class IQ_Option:
             pass
         return self.api.sold_options_respond
 
-    # __________________for Digital___________________
+    def sell_digital_option(self, options_ids):
+        self.api.sell_digital_option(options_ids)
+        self.api.sold_digital_options_respond = None
+        while self.api.sold_digital_options_respond == None:
+            pass
+        return self.api.sold_digital_options_respond
+# __________________for Digital___________________
 
     def get_digital_underlying_list_data(self):
         self.api.underlying_list_data = None
@@ -969,15 +1060,24 @@ class IQ_Option:
             exp).strftime("%Y%m%d%H%M"))
         instrument_id = "do" + active + dateFormated + \
                         "PT" + str(duration) + "M" + action + "SPT"
-        self.api.digital_option_placed_id = None
+        # self.api.digital_option_placed_id = None
 
-        self.api.place_digital_option(instrument_id, amount)
-        while self.api.digital_option_placed_id == None:
+        request_id = self.api.place_digital_option(instrument_id, amount)
+
+        while self.api.digital_option_placed_id.get(request_id) == None:
             pass
-        if isinstance(self.api.digital_option_placed_id, int):
-            return True, self.api.digital_option_placed_id
+        digital_order_id = self.api.digital_option_placed_id.get(request_id)
+        if isinstance(digital_order_id, int):
+            return True, digital_order_id
         else:
-            return False, self.api.digital_option_placed_id
+            return False, digital_order_id
+
+        # while self.api.digital_option_placed_id == None:
+        #     pass
+        # if isinstance(self.api.digital_option_placed_id, int):
+        #     return True, self.api.digital_option_placed_id
+        # else:
+        #     return False, self.api.digital_option_placed_id
 
     def get_digital_spot_profit_after_sale(self, position_id):
         def get_instrument_id_to_bid(data, instrument_id):
@@ -1003,23 +1103,25 @@ class IQ_Option:
         else:
             logging.error(
                 'get_digital_spot_profit_after_sale position error' + str(position["instrument_id"]))
-        ACTIVES = position["instrument_underlying"]
-        amount = max(position["buy_amount"], position["sell_amount"])
+
+        ACTIVES = position['raw_event']['instrument_underlying']
+        amount = max(position['raw_event']["buy_amount"],
+                     position['raw_event']["sell_amount"])
         start_duration = position["instrument_id"].find("PT") + 2
         end_duration = start_duration + \
-                       position["instrument_id"][start_duration:].find("M")
+            position["instrument_id"][start_duration:].find("M")
 
         duration = int(position["instrument_id"][start_duration:end_duration])
         z2 = False
 
-        getAbsCount = position["count"]
-        instrumentStrikeValue = position["instrument_strike_value"] / 1000000.0
-        spotLowerInstrumentStrike = position["extra_data"]["lower_instrument_strike"] / 1000000.0
-        spotUpperInstrumentStrike = position["extra_data"]["upper_instrument_strike"] / 1000000.0
+        getAbsCount = position['raw_event']["count"]
+        instrumentStrikeValue = position['raw_event']["instrument_strike_value"] / 1000000.0
+        spotLowerInstrumentStrike = position['raw_event']["extra_data"]["lower_instrument_strike"] / 1000000.0
+        spotUpperInstrumentStrike = position['raw_event']["extra_data"]["upper_instrument_strike"] / 1000000.0
 
-        aVar = position["extra_data"]["lower_instrument_id"]
-        aVar2 = position["extra_data"]["upper_instrument_id"]
-        getRate = position["currency_rate"]
+        aVar = position['raw_event']["extra_data"]["lower_instrument_id"]
+        aVar2 = position['raw_event']["extra_data"]["upper_instrument_id"]
+        getRate = position['raw_event']["currency_rate"]
 
         # ___________________/*position*/_________________
         instrument_quites_generated_data = self.get_instrument_quites_generated_data(
@@ -1060,7 +1162,7 @@ class IQ_Option:
                       (spotUpperInstrumentStrike - spotLowerInstrumentStrike)) * (f2 - f)
             else:
                 instrumentStrikeValue = (spotUpperInstrumentStrike - instrumentStrikeValue) / (
-                        spotUpperInstrumentStrike - spotLowerInstrumentStrike)
+                    spotUpperInstrumentStrike - spotLowerInstrumentStrike)
                 f -= f2
             f = f2 + (instrumentStrikeValue * f)
 
@@ -1090,7 +1192,8 @@ class IQ_Option:
         self.api.result = None
         while self.get_async_order(position_id)["position-changed"] == {}:
             pass
-        position_changed = self.get_async_order(position_id)["position-changed"]["msg"]
+        position_changed = self.get_async_order(
+            position_id)["position-changed"]["msg"]
         self.api.close_digital_option(position_changed["external_id"])
         while self.api.result == None:
             pass
@@ -1111,7 +1214,8 @@ class IQ_Option:
 
         while self.get_async_order(buy_order_id)["position-changed"] == {}:
             pass
-        order_data = self.get_async_order(buy_order_id)["position-changed"]["msg"]
+        order_data = self.get_async_order(
+            buy_order_id)["position-changed"]["msg"]
         if order_data != None:
             if order_data["status"] == "closed":
                 if order_data["close_reason"] == "expired":
@@ -1271,7 +1375,8 @@ class IQ_Option:
         self.api.position = None
         while self.get_async_order(order_id)["position-changed"] == {}:
             pass
-        position_id = self.get_async_order(order_id)["position-changed"]["msg"]["external_id"]
+        position_id = self.get_async_order(
+            order_id)["position-changed"]["msg"]["external_id"]
         self.api.get_digital_position(position_id)
         while self.api.position == None:
             pass
@@ -1340,11 +1445,11 @@ class IQ_Option:
             return False
 
     def close_position_v2(self, position_id):
-        while self.get_async_order(position_id) is None:
+        while self.get_async_order(position_id) == None:
             pass
         position_changed = self.get_async_order(position_id)
         self.api.close_position(position_changed["id"])
-        while self.api.close_position_data is None:
+        while self.api.close_position_data == None:
             pass
         if self.api.close_position_data["status"] == 2000:
             return True
@@ -1354,7 +1459,7 @@ class IQ_Option:
     def get_overnight_fee(self, instrument_type, active):
         self.api.overnight_fee = None
         self.api.get_overnight_fee(instrument_type, OP_code.ACTIVES[active])
-        while self.api.overnight_fee is None:
+        while self.api.overnight_fee == None:
             pass
         if self.api.overnight_fee["status"] == 2000:
             return True, self.api.overnight_fee["msg"]
@@ -1379,8 +1484,9 @@ class IQ_Option:
         active_id = OP_code.ACTIVES[active]
         self.api.Subscribe_Live_Deal(name, active_id, _type)
         """
-        self.api.live_deal_data[name][active][_type]=deque(list(),buffersize) 
-        
+        self.api.live_deal_data[name][active][_type]=deque(list(),buffersize)
+
+
         while len(self.api.live_deal_data[name][active][_type])==0:
             self.api.Subscribe_Live_Deal(name,active_id,_type)
             time.sleep(1)
@@ -1390,13 +1496,19 @@ class IQ_Option:
         active_id = OP_code.ACTIVES[active]
         self.api.Unscribe_Live_Deal(name, active_id, _type)
         """
-         
+
         while len(self.api.live_deal_data[name][active][_type])!=0:
             self.api.Unscribe_Live_Deal(name,active_id,_type)
             del self.api.live_deal_data[name][active][_type]
             time.sleep(1)
         """
 
+    def set_digital_live_deal_cb(self, cb):
+        self.api.digital_live_deal_cb = cb
+
+    def set_binary_live_deal_cb(self, cb):
+        self.api.binary_live_deal_cb = cb
+
     def get_live_deal(self, name, active, _type):
         return self.api.live_deal_data[name][active][_type]
 
@@ -1433,16 +1545,77 @@ class IQ_Option:
     def get_users_availability(self, user_id):
         self.api.users_availability = None
 
-        while self.api.users_availability is None:
+        while self.api.users_availability == None:
             self.api.Get_Users_Availability(user_id)
             time.sleep(0.2)
         return self.api.users_availability
 
-    def get_trade_result(self):
+    def get_trade_result(self, expiry):
+        startTime = int(time.time())
         while True:
-            if self.api.resultInfo is None:
-                print("Checking result")
-                time.sleep(0.2)
-                continue
+            nowTime = int(time.time())
+            if nowTime - startTime >= expiry * 60 + 10:
+                return {"win": "loose", "profit_amount": 0}
             else:
-                return self.api.resultInfo
+                if self.api.resultInfo is None:
+                    # print("Checking result")
+                    time.sleep(0.2)
+                    continue
+                else:
+                    return self.api.resultInfo
+
+    def drawSupResLine(self, time, price, activeId):
+        self.api.draw_horizontal_line(time, price, activeId)
+
+    def tradeBuy(self, price, ACTIVE, ACTION, expiration):
+        self.api.resultInfo = None
+        self.api.tradeId = None
+        self.api.tradeStatus = None
+        req_id = "buy"
+        self.api.buyv3(price, OP_code.ACTIVES[ACTIVE], ACTION, expiration, req_id)
+        start_t = time.time()
+        self.api.result = None
+        while self.api.tradeStatus == None and self.api.tradeId == None:
+            if time.time() - start_t >= 30:
+                logging.error('**warning** buy late 5 sec')
+                return False, -1
+        return self.api.tradeStatus, self.api.tradeId
+
+    def tradeBuyRawExpiry(self, price, ACTIVE, ACTION, option, expiration):
+        self.api.resultInfo = None
+        self.api.tradeId = None
+        self.api.tradeStatus = None
+        req_id = "buy"
+        self.api.buyv3_by_raw_expired(price, OP_code.ACTIVES[ACTIVE], ACTION, option, expiration, req_id)
+        start_t = time.time()
+        self.api.result = None
+        while self.api.tradeStatus == None and self.api.tradeId == None:
+            if time.time() - start_t >= 30:
+                logging.error('**warning** buy late 5 sec')
+                return False, None
+        return self.api.tradeStatus, self.api.tradeId
+
+    def get_all_open_times(self):
+        # for binary option turbo and binary
+        OPEN_TIME = nested_dict(3, dict)
+        binary_data = self.get_all_init_v2()
+        binary_list = ["binary", "turbo"]
+        for option in binary_list:
+            if binary_data is not None:
+                binaryOptionData = binary_data.get(option)
+                if binaryOptionData is not None:
+                    for actives_id in binaryOptionData.get("actives"):
+                        active = binaryOptionData.get("actives").get(actives_id)
+                        if active is not None:
+                            dottedName = str(active.get("name"))
+                            if dottedName is not None:
+                                name = dottedName.split(".")[1]
+                                if active.get("enabled"):
+                                    if active.get("is_suspended"):
+                                        OPEN_TIME[option][name]["open"] = False
+                                    else:
+                                        OPEN_TIME[option][name]["open"] = True
+                                else:
+                                    OPEN_TIME[option][name]["open"] = active.get("enabled")
+
+        return OPEN_TIME
